# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import os
import sys

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.files.base import ContentFile
from django.core.files.storage import FileSystemStorage
from django.core.management.base import BaseCommand

from ... import js_choices_settings as default_settings
from ...core import generate_js


class Command(BaseCommand):
    help = 'Creates a static choices.js file for django-js-choices'

    def add_arguments(self, parser):
        parser.add_argument('-l', '--locale',
            action='store',
            dest='locale',
            default=settings.LANGUAGE_CODE,
            help='Pass custom locale to generate choices file',
        )

    def get_location(self):
        output_path = getattr(settings, 'JS_CHOICES_OUTPUT_PATH', default_settings.JS_OUTPUT_PATH)
        if output_path:
            return output_path
        if not hasattr(settings, 'STATIC_ROOT') or not settings.STATIC_ROOT:
            raise ImproperlyConfigured('The collectstatic_js_choices command needs settings.JS_CHOICES_OUTPUT_PATH '
                                       'or settings.STATIC_ROOT to be set.')
        return os.path.join(settings.STATIC_ROOT, 'django_js_choices', 'js')

    def handle(self, *args, **options):
        locale = options['locale']
        location = self.get_location()
        file = 'choices-{}.js'.format(locale)
        fs = FileSystemStorage(location=location)
        if fs.exists(file):
            fs.delete(file)
        content = generate_js(locale)
        fs.save(file, ContentFile(content))
        if len(sys.argv) > 1 and sys.argv[1] in ['collectstatic_js_choices']:
            self.stdout.write('{0} file written to {1}'.format(file, location))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django_js_choices.core import generate_js
from django.http import HttpResponse


def choices_js(request):
    locale = request.GET.get('lang', None) or\
        request.GET.get('locale', None)
    response_body = generate_js(locale)
    return HttpResponse(response_body, **{'content_type': 'application/javascript'})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json

from django.apps import apps
from django.conf import settings
from django.template import loader
from django.utils.encoding import force_text
from django.utils.translation import activate, deactivate

from . import js_choices_settings as default_settings
from . import rjsmin


def prepare_choices(choices):
    new_choices = []
    for choice in choices:
        if len(choice) != 2:
            continue
        try:
            json.dumps(choice[0])
            new_choices.append((choice[0], force_text(choice[1])))
        except TypeError:
            new_choices.append((force_text(choice[0]), force_text(choice[1])))
    return new_choices


def generate_js(locale=None):
    raw_choices = []
    named_choices = {}
    if locale:
        activate(locale)
    for app_config in apps.get_app_configs():
        for model in app_config.get_models():
            for field in model._meta.get_fields():
                choices = getattr(field, 'flatchoices', None)
                if choices:
                    name = '{}_{}'.format(
                        model._meta.label_lower.replace('.', '_'),
                        field.name
                    )
                    value = json.dumps(dict(tuple(prepare_choices(choices))))
                    try:
                        index = raw_choices.index(value)
                    except ValueError:
                        index = len(raw_choices)
                        raw_choices.append(value)
                    finally:
                        named_choices[name] = index
    if locale:
        deactivate()
    js_var_name = getattr(settings, 'JS_CHOICES_JS_VAR_NAME', default_settings.JS_VAR_NAME)
    js_global_object_name = getattr(settings, 'JS_CHOICES_JS_GLOBAL_OBJECT_NAME', default_settings.JS_GLOBAL_OBJECT_NAME)
    minfiy = getattr(settings, 'JS_CHOICES_JS_MINIFY', default_settings.JS_MINIFY)
    js_content = loader.render_to_string('django_js_choices/choices_js.tpl', {
        'raw_choices_list': raw_choices,
        'named_choices': json.dumps(named_choices),
        'js_var_name': js_var_name,
        'js_global_object_name': js_global_object_name,
    })
    if minfiy:
        js_content = rjsmin.jsmin(js_content)
    return js_content



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import codecs
import os
from distutils.core import setup

from setuptools import find_packages


def read(*parts):
    filename = os.path.join(os.path.dirname(__file__), *parts)
    with codecs.open(filename, encoding='utf-8') as fp:
        return fp.read()


version_tuple = __import__('django_js_choices').VERSION
version = '.'.join([str(v) for v in version_tuple])
setup(
    name='django-js-choices',
    version=version,
    description='Javascript model field\'s choices handling for Django.',
    long_description=read('README.rst'),
    url='https://github.com/lorinkoz/django-js-choices',
    download_url='http://pypi.python.org/pypi/django-js-choices/',
    author='Lorenzo Peña',
    author_email='lorinkoz@gmail.com',
    license='MIT',
    classifiers=[
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Framework :: Django',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
    ],
    keywords='django javascript model field choices',
    packages=find_packages(),
    package_data={
        'django_js_choices': [
            'templates/django_js_choices/*',
        ]
    },
    install_requires=[
        'Django>=1.5',
    ]
)
